use diesel;
use diesel::prelude::*;
use diesel::sqlite::SqliteConnection;

use std::env;

use crate::models::{Image, Tag};
use crate::schema::images::dsl::*;
use crate::schema::tags::dsl::*;

type Conn = SqliteConnection;

pub fn establish_connection() -> Conn {
    let database_url = env::var("DATABASE_URL").expect("DATABASE_URL must be set");
    SqliteConnection::establish(&database_url)
        .expect(&format!("Error connecting to {}", database_url))
}

pub fn load_images(conn: &Conn, start: i64, limit: i64) -> Vec<Image> {
    images
        .filter(published.eq(false))
        .offset(start)
        .limit(limit)
        .load::<Image>(conn)
        .expect("Error loading images")
}

pub fn load_tags<T>(conn: &Conn, uuid: T) -> Vec<Tag>
where
    T: Into<String>,
{
    tags.filter(image_id.eq(uuid.into()))
        .load::<Tag>(conn)
        .expect("Error loading tags.")
}

pub fn save_image(conn: &Conn, img: &Image) -> QueryResult<usize> {
    diesel::insert_into(images).values(img).execute(conn)
}

pub fn save_tags(conn: &Conn, tag: &Tag) -> QueryResult<usize> {
    diesel::insert_into(tags).values(tag).execute(conn)
}

pub fn delete_image<T>(conn: &Conn, uuid: T) -> QueryResult<usize>
where
    T: Into<String>,
{
    let uuid_str = uuid.into();
    diesel::delete(tags.filter(image_id.eq(&uuid_str))).execute(conn)?;
    diesel::delete(images.filter(id.eq(&uuid_str))).execute(conn)
}
